/*
 * $Id: common.c,v 0.1 2007/04/02 18:20:11 xenion Exp $
 *
 * Copyright (c) 2007 Dallachiesa Michele <micheleDOTdallachiesaATposteDOTit> 
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */


#define _GNU_SOURCE

#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <signal.h>
#include <errno.h>
#include <stdlib.h>
#include <time.h>
#include "common.h"


/* globals */


static int debug = 0;
static FILE *f = NULL;


/* extern */

extern void cleanup();


/*******************************************/


void enable_debug()
{
  debug = 1;
}


void disable_debug()
{
  debug = 0;
}


void open_logfile(char *pathname)
{
  if(f)
    SAFE_FCLOSE(f);

  if((f =  fopen(pathname, "w")) == NULL)
    FATAL("fopen(): %s", strerror(errno));
}

void close_logfile()
{
  if(f != NULL && f != stdout && f != stderr)
    SAFE_FCLOSE(f);
}


void
logthis(char *file, const char *function, int line, int ifdebug,int h, int n, const char *fmt, ...)
{
  static int after_h = 0;


  va_list         ap;

  // The program must first execute the macro va_start within the body
  // of the function to initialize an object with context information.

  va_start(ap, fmt);

  if(ifdebug && debug == 0)
    return;

  if(f == NULL)
    f = stdout;


  if(after_h)
    h = 0;

  if(h)
    {
      if(debug)
        fprintf(f, "%s %s:%d: ",strtime(time(NULL)),function, line);
    }

  vfprintf(f,fmt, ap);
  if(n)
    fprintf(f, "\n");
  fflush(f);

  va_end(ap);

  if(n)
    after_h = 0;
  else
    after_h = 1;
}


void
fatal(char *file, const char *function, int line, const char *fmt, ...)
{
  va_list         ap;


  // The program must first execute the macro va_start within the body
  // of the function to initialize an object with context information.

  va_start(ap, fmt);

  printf("\nfatal error at %s:%s:%d: ",file, function, line);
  vprintf(fmt, ap);
  printf("; exit forced.\n\n");

  va_end(ap);

  sig_unlock(); // if lock, unlock. else, nothing changes.
  raise(SIGUSR1);
}


static void
sigdie(int signo)
{

  LOG(0,0,"\n--\n");

  LOG(1,1,"Caught %s signal (%d), cleaning up.\n", SIG_NAME(signo), signo);

  cleanup();

  // questa deve essere l'ultima cosa prima della exit.
  close_logfile();

  LOG(1,1,"");
  LOG(1,1,"Bye!");
  LOG(1,1,"");

  exit(0);
}


void init_sighandlers()
{
  signal(SIGSEGV, sigdie);
  signal(SIGTERM, sigdie);
  signal(SIGINT, sigdie);
  signal(SIGUSR1, sigdie);
}


void sig_lock()
{
  signal(SIGTERM, SIG_IGN);
  signal(SIGINT, SIG_IGN);
  signal(SIGUSR1, SIG_IGN);
}


void sig_unlock()
{
  signal(SIGTERM, sigdie);
  signal(SIGINT, sigdie);
  signal(SIGUSR1, sigdie);
}


void
print_mem(u_int8_t *p, u_int32_t len, u_int32_t cols)
{
  u_int32_t off,line,col;

  printf("+ Dumping memory from %p for %d bytes %s\n",p,len, cols == 1 ? "(hex bin dec chr)" : "(hex)");

  for(off=0,col=0,line=0;off<len;off++)
    {
      if(col == 0)
        {
          printf("%s %p+%-8d",line>0?"\n|":"|",p,off);
        }

      if(cols == 1)
        {
          printf("0x%.2x %d%d%d%d%d%d%d%d %.3d %s",p[off],p[off] & 0x80 ? 1 : 0,p[off] & 0x40 ? 1 : 0,p[off] & 0x20 ? 1 : 0,p[off] & 0x10 ? 1 : 0,p[off] & 0x08 ? 1 : 0,p[off] & 0x04 ? 1 : 0,p[off] & 0x02 ? 1 : 0,p[off] & 0x01 ? 1 : 0, p[off], str_char(p[off]));
        }
      else
//        printf(" %.2x",p[off]);
//        printf(" %.2x:%s",p[off], str_char(p[off]));
        printf(" :%s", str_char(p[off]));


      if(++col >= cols)
        {
          col = 0;
          line++;
        }
    }
  printf("\n+ End\n");
}


char *str_char(unsigned char c)
{
  static char s[8];

  if(c >= 32 && 126)
    {
      sprintf(s,"'%c' ", c);
      return s;
    }

  switch(c)
    {
    case '\0':
      return "'\\0'";
    case '\r':
      return "'\\r'";
    case '\n' :
      return "'\\n'";
    default:
      sprintf(s,"?%d", c);
      return s;
    }

}


char *
strtime(time_t t)
{
  struct tm      *mytm;
  static char s[20];


  mytm = localtime(&t);
  strftime(s, 20, "%d/%m/%Y#%H:%M:%S", mytm);
  return s;
}



/* EOF*/

